"use strict";
// Copyright 2016-2018, Pulumi Corporation.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
Object.defineProperty(exports, "__esModule", { value: true });
const eventhub = require("@pulumi/azure/eventhub");
const pulumi = require("@pulumi/pulumi");
const subscriptionMod = require("../subscription");
/**
 * Creates a new subscription to the given blob using the callback provided, along with optional
 * options to control the behavior of the subscription.
 */
function onTopicEvent(name, namespace, topic, args, opts) {
    args = args || {};
    let subscription = args.subscription;
    if (!subscription) {
        subscription = new eventhub.Subscription(name, {
            resourceGroupName: pulumi.output(args.resourceGroup).apply(g => g.name),
            namespaceName: topic.namespaceName,
            topicName: topic.name,
            maxDeliveryCount: pulumi.output(args.maxDeliveryCount).apply(c => c === undefined ? 10 : c),
        }, { parent: topic });
    }
    // The topic binding does not store the storage connection string directly.  Instead, the
    // connection string is put into the app settings (under whatever key we want). Then, the
    // .connection property of the binding contains the *name* of that app setting key.
    const bindingConnectionKey = "BindingConnectionAppSettingsKey";
    const bindings = pulumi.all([topic.name, subscription.name]).apply(([topicName, subscriptionName]) => {
        const topicBinding = {
            name: "topic",
            direction: "in",
            type: "serviceBusTrigger",
            topicName: topicName,
            subscriptionName: subscriptionName,
            connection: bindingConnectionKey,
        };
        return [topicBinding];
    });
    // Place the mapping from the well known key name to the storage account connection string in
    // the 'app settings' object.
    const appSettingsOutput = args.appSettings || pulumi.output({});
    args.appSettings = pulumi.all([appSettingsOutput, namespace.defaultPrimaryConnectionString]).apply(([appSettings, connectionString]) => {
        appSettings[bindingConnectionKey] = connectionString;
        return appSettings;
    });
    return new TopicEventSubscription(name, namespace, topic, subscription, bindings, args, opts);
}
exports.onTopicEvent = onTopicEvent;
class TopicEventSubscription extends subscriptionMod.EventSubscription {
    constructor(name, namespace, topic, subscription, bindings, args, options) {
        super("azure-serverless:topic:TopicEventSubscription", name, bindings, args, options);
        this.namespace = namespace;
        this.topic = topic;
        this.subscription = subscription;
    }
}
exports.TopicEventSubscription = TopicEventSubscription;
//# sourceMappingURL=topic.js.map