"use strict";
// Copyright 2016-2018, Pulumi Corporation.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
function __export(m) {
    for (var p in m) if (!exports.hasOwnProperty(p)) exports[p] = m[p];
}
Object.defineProperty(exports, "__esModule", { value: true });
const azure = require("@pulumi/azure");
const pulumi = require("@pulumi/pulumi");
const util_1 = require("./util");
__export(require("azure-functions-ts-essentials"));
function blobContent(name, handler) {
    return __awaiter(this, void 0, void 0, function* () {
        const serializedHandler = yield pulumi.runtime.serializeFunction(handler);
        const map = {};
        map["host.json"] = new pulumi.asset.StringAsset(JSON.stringify({}));
        map[`${name}/function.json`] = new pulumi.asset.StringAsset(JSON.stringify({
            "disabled": false,
            "bindings": [
                {
                    "authLevel": "anonymous",
                    "type": "httpTrigger",
                    "direction": "in",
                    "name": "req",
                },
                {
                    "type": "http",
                    "direction": "out",
                    "name": "$return",
                },
            ],
        }));
        map[`${name}/index.js`] = new pulumi.asset.StringAsset(`module.exports = require("./handler").handler`),
            map[`${name}/handler.js`] = new pulumi.asset.StringAsset(serializedHandler.text);
        return map;
    });
}
// An Azure function exposed via an HTTP endpoint.
class HttpFunction extends pulumi.ComponentResource {
    constructor(name, handler, options) {
        super("azure:HttpFunction", name, {}, options);
        const parentArgs = { parent: this };
        this.resourceGroup = new azure.core.ResourceGroup(`${name}-rg`, {
            location: "West US 2",
        }, parentArgs);
        const resourceGroupArgs = {
            resourceGroupName: this.resourceGroup.name,
            location: this.resourceGroup.location,
        };
        this.storageAccount = new azure.storage.Account(`${name}sa`, Object.assign({}, resourceGroupArgs, { accountKind: "StorageV2", accountTier: "Standard", accountReplicationType: "LRS" }), parentArgs);
        this.storageContainer = new azure.storage.Container(`${name}-c`, {
            resourceGroupName: this.resourceGroup.name,
            storageAccountName: this.storageAccount.name,
            containerAccessType: "private",
        }, parentArgs);
        this.blob = new azure.storage.ZipBlob(`${name}-b`, {
            resourceGroupName: this.resourceGroup.name,
            storageAccountName: this.storageAccount.name,
            storageContainerName: this.storageContainer.name,
            type: "block",
            content: new pulumi.asset.AssetArchive(blobContent(name, handler)),
        }, parentArgs);
        this.codeBlobUrl = util_1.signedBlobReadUrl(this.blob, this.storageAccount, this.storageContainer);
        this.appServicePlan = new azure.appservice.Plan(`${name}-p`, Object.assign({}, resourceGroupArgs, { kind: "FunctionApp", 
            // https://social.msdn.microsoft.com/Forums/azure/en-US/665c365d-2b86-4a77-8cea-72ccffef216c
            sku: {
                tier: "Dynamic",
                size: "Y1",
            } }), parentArgs);
        this.functionApp = new azure.appservice.FunctionApp(`${name}-app`, Object.assign({}, resourceGroupArgs, { appServicePlanId: this.appServicePlan.id, storageConnectionString: this.storageAccount.primaryConnectionString, appSettings: {
                "WEBSITE_RUN_FROM_ZIP": this.codeBlobUrl,
            } }), parentArgs);
        this.endpoint = this.functionApp.defaultHostname.apply(h => {
            return `https://${h}/api/${name}`;
        });
    }
}
exports.HttpFunction = HttpFunction;
//# sourceMappingURL=functionApp.js.map